% Tripod input with Pengcheng scordablature output.
% Score and MIDI.

\include "tripod-data.ly"

% Treat the tripod input as 60-equal
% although it isn't output to MIDI
semitoeSize = #3/10
inchSize = #1/10

#(set-tripod-names semitoeSize semitoeSize inchSize inchSize)

% now reset the output scale
majorScale = #'#(0 1 2 5/2 7/2 9/2 11/2)
#(ly:set-default-scale (ly:make-scale majorScale))

% How tripod nominals map to 12 Halberstadt keys
% (scale-degree . alteration)
tripodKeys = #'#(
    (0 . 0) ; C
    (1 . 0) ; D
    (2 . 0) ; E
    (3 . 1/2) ; F#
    (4 . 1/2) ; G#
    (6 . -1/2) ; Bb
)

% Pitches that are allowed to change.
% ((old degree . old alteration) . (new degree . new alteration))
substitutions = #'(
    ((3 . 1) . (4 . 0)) ; Fx -> G
    ((4 . 1) . (5 . 0)) ; Gx -> A
    ((6 . -1) . (5 . 0)) ; Bbb -> A
)

% The magic function that converts a tripod pitch
% to a scordablature pitch.
% Transposes from register 3 to that specified.
#(define (pengcheng-pitch register p)
  (let* ((reg (+ (ly:pitch-octave p) register -3))
         (semitoes (/ (ly:pitch-alteration p) semitoeSize))
         (tripod-degree (ly:pitch-notename p))
          ; There are 9 tripod degrees to an acoustic octave.
          ; There are 6 tripod degrees to a keyboard octave.
          ; Yan, the zero point, is F# above middle C.
          ; So keyboard middle C is tripod degree -3.
          ; Middle C is zero.
         (absolute-degree (+ tripod-degree (* 9 reg) 3))
         (peng-reg (floor (/ absolute-degree 6)))
         (tripod-key (- absolute-degree (* 6 peng-reg)))
         (base-pitch (vector-ref tripodKeys tripod-key))
         (peng-degree (car base-pitch))
         (alteration (+ (cdr base-pitch) (/ semitoes 2)))
         (new-pair (assoc (cons peng-degree alteration) substitutions)))
    (if (eq? new-pair #f)
        (ly:make-pitch peng-reg peng-degree alteration)
        (ly:make-pitch peng-reg (cadr new-pair) (cddr new-pair)))))

% Convert all notes.
% This is copied from "Transposing music with minimum accidentals"
% In the "Pitches" snippets in the documentation.
#(define (tripod-to-pengcheng register music)
  (let* ((es (ly:music-property music 'elements))
         (e (ly:music-property music 'element))
         (p (ly:music-property music 'pitch)))
    (if (pair? es)
       (ly:music-set-property!
         music 'elements
         (map (lambda (x) (tripod-to-pengcheng register x)) es)))
    (if (ly:music? e)
       (ly:music-set-property!
         music 'element
         (tripod-to-pengcheng register e)))
    (if (ly:pitch? p)
       (begin
         (set! p (pengcheng-pitch register p))
         (ly:music-set-property! music 'pitch p)))
    music))

% Convert to Pengcheng pitches,
% and correct for a previous transposition to register
pengcheng = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (tripod-to-pengcheng (- 7 register) m))

% Tripod note names in a given register, ignored here.
% Also does the Pengcheng conversion.
tripod = #(define-music-function (parser location register m)
  (integer? ly:music?)
  (tripod-to-pengcheng 4 m))

\paper{
    newStaffSize = #22
    #(define fonts (make-pango-font-tree "URW Bookman L" "Sans" "Mono"
                    (/ newStaffSize 20)))
}
lyricsize=\tiny

% All clefs are played as treble,
% assuming the music in them is appropriately transposed.
% I can't work out how to automatically indicate
% that the pitch output should be different.
fourfiveclef = {
  \clef treble
}
fourclef = {
  \clef treble
}
threeclef = {
  \clef treble
}
